/* $Id: cpl_filter.h,v 1.16 2010/05/03 11:15:25 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2010/05/03 11:15:25 $
 * $Revision: 1.16 $
 * $Name: cpl-5_2_0 $
 */

#ifndef CPL_FILTER_H
#define CPL_FILTER_H

#include <cpl_macros.h>

CPL_BEGIN_DECLS

/*-----------------------------------------------------------------------------
                                   New types
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @ingroup cpl_image_filter
 * 
 * @brief These are the supported border modes. For a kernel/mask of width
 * 2n+1, the n left- and rightmost image columns do not have raw pixels for
 * the whole kernel/mask. The same holds for the top and bottom image rows.
 * The border mode defines the filtering of such border pixels.
 * 
 *   <table class="ec" align="center">
 *     <tr>
 *       <td class="ecl">CPL_BORDER_FILTER</td>
 *       <td class="ecr">
 *         Filter the border using the reduced
 *     number of pixels. If in median filtering the
 *     number of pixels is even choose any one of the
 *     two central values.
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_BORDER_CROP</td>
 *       <td class="ecr">
 *        Crop the filtered image. 
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_BORDER_NOP</td>
 *       <td class="ecr">
 *         Do not modify the border of the filtered image.
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_BORDER_COPY</td>
 *       <td class="ecr">
 *         Copy the border of the raw image
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_BORDER_ZERO</td>
 *       <td class="ecr">
 *        Set the border of the filtered mask to zero 
 *       </td>
 *     </tr>
 *   </table>
 *
 * 
 */
/*----------------------------------------------------------------------------*/

enum _cpl_border_mode_
{
    CPL_BORDER_FILTER,
    /*
      Filter the border.
    */

    CPL_BORDER_ZERO,
    /*
      Set the border to zero.
    */

    CPL_BORDER_CROP,
    /*
      Crop filtered image.
    */

    CPL_BORDER_NOP,
    /*
      Do not modify the border of the filtered image.
    */

    CPL_BORDER_COPY
    /*
      Copy the border of the raw image.
    */

};

/**
 * @ingroup cpl_image_filter
 *
 * @brief
 *   The border mode type.
 */

typedef enum _cpl_border_mode_ cpl_border_mode;


/*----------------------------------------------------------------------------*/
/**
 * 
   @ingroup cpl_image_filter
 * 
 * @brief These are the supported filter modes.
 * 
 * 
 *   <table class="ec" align="center">
 *     <tr>
 *       <td class="ecl">CPL_FILTER_LINEAR</td>
 *       <td class="ecr">
 *        A linear filter. The kernel elements are used as weights
 *        like this:
 *
    @verbatim
    Kernel 1 2 3    Image        ...
           4 5 6         ... 1.0 2.0 3.0 ...
           7 8 9         ... 4.0 5.0 6.0 ...
                         ... 7.0 8.0 9.0 ...
                                 ...
    @endverbatim
 * 
 *        The filtered value corresponding to the pixel whose value is 5.0 is:
 *        \f$\frac{(1*1.0+2*2.0+3*3.0+4*4.0+5*5.0+6*6.0+7*7.0+8*8.0+9*9.0)}{1+2+3+4+5+6+7+8+9}\f$
 *
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_AVERAGE</td>
 *       <td class="ecr">
 *    An average filter, i.e. the output pixel is the
 *    arithmetic average of the surrounding (1 + 2 * hsizex)
 *    * (1 + 2 * hsizey) pixels.
 *    The cost per pixel is O(hsizex*hsizey).
 *    The two images may have different pixel types.
 *    When the input and output pixel types
 *    are identical, the arithmetic is done with that type,
 *    e.g. int for two integer images. When the input and
 *    output pixel types differ, the arithmetic is done in
 *    double precision when one of the two images have
 *    pixel type CPL_TYPE_DOUBLE, otherwise float is used.
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_AVERAGE_FAST</td>
 *       <td class="ecr">
 *    The same as CPL_FILTER_AVERAGE, except that it uses a running
 *    average, which will lead to a significant loss of precision if
 *    there are large differences in the magnitudes of the input pixels.
 *    The cost per pixel is O(1) if all elements in the kernel are used,
 *    otherwise the filtering is done as for CPL_FILTER_AVERAGE.
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_MEDIAN</td>
 *       <td class="ecr">
 *        A median filter. The pixel types of the input and
 *        output images must be identical. All border modes are
 *        supported.
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_STDEV</td>
 *       <td class="ecr">
 *     The filtered value is the standard deviation of the included
 *     input pixels.
 *
 *
       @verbatim
       mask                  Image        ...
              1   0   1           ... 1.0 2.0 3.0 ...
              0   1   0           ... 4.0 5.0 6.0 ...
              1   0   1           ... 7.0 8.0 9.0 ...
                                          ...
       @endverbatim
 *
 *     The pixel with value 5.0 will have a filtered value of:
 *     std_dev(1.0, 3.0, 5.0, 7.0, 9.0)
 *
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_STDEV_FAST</td>
 *       <td class="ecr">
 *     The same as CPL_FILTER_STDEV, except that it uses the same running
 *     method employed in CPL_FILTER_AVERAGE_FAST, which will lead to a
 *     significant loss of precision if there are large differences in the
 *     magnitudes of the input pixels. As for CPL_FILTER_AVERAGE_FAST, the cost
 *     per pixel is O(1) if all elements are used, otherwise the filtering is
 *     done as for CPL_FILTER_STDEV.
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_MORPHO</td>
 *       <td class="ecr">
 *        A Morphological filter. The kernel elements are used as weights
 *        on the sorted values covered by the kernel:
 *
    @verbatim
    Kernel 1 2 3    Image        ...
           4 5 6         ... 4.0 6.0 5.0 ...
           7 8 9         ... 3.0 1.0 2.0 ...
                         ... 7.0 8.0 9.0 ...
                                 ...
    @endverbatim
 *
 *  The filtered value corresponding to the pixel whose value is 5.0 is:
 *  \f$\frac{(1*1.0+2*2.0+3*3.0+4*4.0+5*5.0+6*6.0+7*7.0+8*8.0+9*9.0)}{1+2+3+4+5+6+7+8+9}\f$
 *
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_EROSION</td>
 *       <td class="ecr">
 *        The binary erosion filter. This filter is used in mask filtering.
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_DILATION</td>
 *       <td class="ecr">
 *        The binary dilation filter. This filter is used in mask filtering.
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_OPENING</td>
 *       <td class="ecr">
 *        The binary opening filter. This filter is used in mask filtering.
 *       </td>
 *     </tr>
 *     <tr>
 *       <td class="ecl">CPL_FILTER_CLOSING</td>
 *       <td class="ecr">
 *        The binary closing filter. This filter is used in mask filtering.
 *       </td>
 *     </tr>
 *   </table>
 *
 * 
 */
/*----------------------------------------------------------------------------*/


enum _cpl_filter_mode_
{

    CPL_FILTER_EROSION,

    /*
      The erosion filter for binary images (cpl_mask)
    */

    CPL_FILTER_DILATION,

    /*
      The dilation filter for binary images (cpl_mask)
    */

    CPL_FILTER_OPENING,

    /*
      The opening filter for binary images (cpl_mask)
    */

    CPL_FILTER_CLOSING,

    /*
      The closing filter for binary images (cpl_mask)
    */

    CPL_FILTER_LINEAR,
    /*    
          A linear filter.
    */

    CPL_FILTER_AVERAGE,
    /*
      An average filter.
    */

    CPL_FILTER_AVERAGE_FAST,
    /*
      A fast average filter.
    */

    CPL_FILTER_MEDIAN,
    /*    
          A median filter.
    */

    CPL_FILTER_STDEV,

    /*
         A standard deviation filter.
    */

    CPL_FILTER_STDEV_FAST,

    /*
         A fast standard deviation filter.
    */

    CPL_FILTER_MORPHO
    /*    
          A Morphological filter.
    */


};

/**
 * @ingroup cpl_image_filter
 *
 * @brief
 *   The filter mode type.
 */

typedef enum _cpl_filter_mode_ cpl_filter_mode;

CPL_END_DECLS

#endif 
